@@ -1,8 +1,26 @@
-import astropy as ap
+import copy
+import re
+from typing import Union
+
+import dask
+import dask.array as da
+import numpy as np
+import psutil
+import xarray as xr
 from astropy import units as u
 from astropy.io import fits
 from astropy.time import Time
-from ..common import (
+
+from xradio._utils.coord_math import _deg_to_rad
+from xradio._utils.dict_helpers import (
+    make_quantity,
+    make_spectral_coord_reference_dict,
+    make_skycoord_dict,
+    make_time_measure_dict,
+)
+
+from xradio.measurement_set._utils._utils.stokes_types import stokes_types
+from xradio.image._util.common import (
     _compute_linear_world_values,
     _compute_velocity_values,
     _compute_world_sph_dims,
@@ -15,41 +33,45 @@ from ..common import (
     _image_type,
     _l_m_attr_notes,
 )
-from xradio._utils.coord_math import _deg_to_rad
-from xradio._utils.dict_helpers import make_quantity
-import copy
-import dask
-import dask.array as da
-import numpy as np
-import re
-from typing import Union
-import xarray as xr
 
 
 def _fits_image_to_xds(
-    img_full_path: str, chunks: dict, verbose: bool, do_sky_coords: bool
+    img_full_path: str,
+    chunks: dict,
+    verbose: bool,
+    do_sky_coords: bool,
+    compute_mask: bool,
 ) -> dict:
     """
+    compute_mask : bool, optional
+        If True (default), compute and attach valid data masks to the xds.
+        If False, skip mask generation for performance. It is solely the responsibility
+        of the user to ensure downstream apps can handle NaN values; do not
+        ask package developers to add this non-standard behavior.
+
     TODO: complete documentation
     Create an xds without any pixel data from metadata from the specified FITS image
     """
-    # memmap = True allows only part of data to be loaded into memory
     # may also need to pass mode='denywrite'
     # https://stackoverflow.com/questions/35759713/astropy-io-fits-read-row-from-large-fits-file-with-mutliple-hdus
-    hdulist = fits.open(img_full_path, memmap=True)
-    attrs, helpers, header = _fits_header_to_xds_attrs(hdulist)
-    hdulist.close()
-    # avoid keeping reference to mem-mapped fits file
-    del hdulist
+    try:
+        hdulist = fits.open(img_full_path, memmap=True)
+        attrs, helpers, header = _fits_header_to_xds_attrs(hdulist, compute_mask)
+    finally:
+        hdulist.close()
+        # avoid keeping reference to mem-mapped fits file
+        del hdulist
     xds = _create_coords(helpers, header, do_sky_coords)
     sphr_dims = helpers["sphr_dims"]
     ary = _read_image_array(img_full_path, chunks, helpers, verbose)
     dim_order = _get_xds_dim_order(sphr_dims)
-    xds = _add_sky_or_aperture(xds, ary, dim_order, helpers, sphr_dims)
+    xds = _add_sky_or_aperture(xds, ary, dim_order, header, helpers, sphr_dims)
     xds.attrs = attrs
     xds = _add_coord_attrs(xds, helpers)
     if helpers["has_multibeam"]:
         xds = _do_multibeam(xds, img_full_path)
+    elif "beam" in helpers and helpers["beam"] is not None:
+        xds = _add_beam(xds, helpers)
     return xds
 
 
@@ -64,12 +86,7 @@ def _add_coord_attrs(xds: xr.Dataset, helpers: dict) -> xr.Dataset:
 
 def _add_time_attrs(xds: xr.Dataset, helpers: dict) -> xr.Dataset:
     time_coord = xds.coords["time"]
-    meta = copy.deepcopy(helpers["obsdate"])
-    del meta["value"]
-    # meta['format'] = 'MJD'
-    # meta['time_scale'] = meta['refer']
-    # del meta['refer']
-    time_coord.attrs = meta
+    time_coord.attrs = copy.deepcopy(helpers["obsdate"]["attrs"])
     xds.assign_coords(time=time_coord)
     return xds
 
@@ -79,13 +96,14 @@ def _add_freq_attrs(xds: xr.Dataset, helpers: dict) -> xr.Dataset:
     meta = {}
     if helpers["has_freq"]:
         meta["rest_frequency"] = make_quantity(helpers["restfreq"], "Hz")
-        meta["frame"] = helpers["specsys"]
-        meta["units"] = "Hz"
+        meta["rest_frequencies"] = [meta["rest_frequency"]]
         meta["type"] = "frequency"
-        meta["wave_unit"] = "mm"
+        meta["wave_units"] = "mm"
         freq_axis = helpers["freq_axis"]
-        meta["crval"] = helpers["crval"][freq_axis]
-        meta["cdelt"] = helpers["cdelt"][freq_axis]
+        meta["reference_frequency"] = make_spectral_coord_reference_dict(
+            helpers["crval"][freq_axis], "Hz", helpers["specsys"]
+        )
+        # meta["cdelt"] = helpers["cdelt"][freq_axis]
     if not meta:
         # this is the default frequency information CASA creates
         meta = _default_freq_info()
@@ -112,26 +130,12 @@ def _add_l_m_attrs(xds: xr.Dataset, helpers: dict) -> xr.Dataset:
     for c in ["l", "m"]:
         if c in xds.coords:
             xds[c].attrs = {
-                "crval": 0.0,
-                "cdelt": helpers[c]["cdelt"],
-                "units": "rad",
-                "type": "quantity",
                 "note": attr_note[c],
             }
     return xds
 
 
 def _add_lin_attrs(xds: xr.Dataset, helpers: dict) -> xr.Dataset:
-    """
-    if helpers["sphr_dims"]:
-        for i, name in zip(helpers["dir_axes"], helpers["sphr_axis_names"]):
-            meta = {
-                "units": "rad",
-                "crval": helpers["crval"][i],
-                "cdelt": helpers["cdelt"][i],
-            }
-            xds.coords[name].attrs = meta
-    """
     if not helpers["sphr_dims"]:
         for i, j in zip(helpers["dir_axes"], ("u", "v")):
             meta = {
@@ -161,33 +165,46 @@ def _xds_direction_attrs_from_header(helpers: dict, header) -> dict:
     direction["projection"] = p0
     helpers["projection"] = p0
     ref_sys = header["RADESYS"]
-    ref_eqx = header["EQUINOX"]
+    ref_eqx = None if ref_sys.upper() == "ICRS" else header["EQUINOX"]
     if ref_sys == "FK5" and ref_eqx == 2000:
         ref_eqx = "J2000.0"
     helpers["ref_sys"] = ref_sys
     helpers["ref_eqx"] = ref_eqx
     # fits does not support conversion frames
-    direction["reference"] = {
-        "type": "sky_coord",
-        "frame": ref_sys,
-        "equinox": ref_eqx,
-        "units": ["rad", "rad"],
-        "value": [0.0, 0.0],
-    }
+    direction["reference"] = make_skycoord_dict([0.0, 0.0], units="rad", frame=ref_sys)
     dir_axes = helpers["dir_axes"]
+    ddata = []
+    dunits = []
     for i in dir_axes:
         x = helpers["crval"][i] * u.Unit(_get_unit(helpers["cunit"][i]))
         x = x.to("rad")
-        direction["reference"]["value"][i] = x.value
+        ddata.append(x.value)
+        # direction["reference"]["value"][i] = x.value
         x = helpers["cdelt"][i] * u.Unit(_get_unit(helpers["cunit"][i]))
-        x = x.to("rad")
-    direction["latpole"] = make_quantity(header["LATPOLE"] * _deg_to_rad, "rad")
-    direction["longpole"] = make_quantity(header["LONPOLE"] * _deg_to_rad, "rad")
+        dunits.append("rad")
+    direction["reference"] = make_skycoord_dict(ddata, units=dunits, frame=ref_sys)
+    if ref_eqx is not None:
+        direction["reference"]["attrs"]["equinox"] = ref_eqx.lower()
+    direction["latpole"] = make_quantity(
+        header["LATPOLE"] * _deg_to_rad, "rad", dims=["l", "m"]
+    )
+    direction["lonpole"] = make_quantity(
+        header["LONPOLE"] * _deg_to_rad, "rad", dims=["l", "m"]
+    )
     pc = np.zeros([2, 2])
     for i in (0, 1):
         for j in (0, 1):
             # dir_axes are now 0-based, but fits needs 1-based
-            pc[i][j] = header[f"PC{dir_axes[i]+1}_{dir_axes[j]+1}"]
+            try:
+                pc[i][j] = header[f"PC{dir_axes[i]+1}_{dir_axes[j]+1}"]
+            except KeyError:
+                try:
+                    pc[i][j] = header[f"PC0{dir_axes[i]+1}_0{dir_axes[j]+1}"]
+                except KeyError:
+                    raise RuntimeError(
+                        f"Could not find PC{dir_axes[i]+1}_{dir_axes[j]+1} or "
+                        f"PC0{dir_axes[i]+1}_0{dir_axes[j]+1} in FITS header"
+                    )
     direction["pc"] = pc
     # Is there really no fits header parameter for projection_parameters?
     direction["projection_parameters"] = np.array([0.0, 0.0])
@@ -232,18 +249,48 @@ def _get_telescope_metadata(helpers: dict, header) -> dict:
         r = np.sqrt(np.sum(xyz * xyz))
         lat = np.arcsin(z / r)
         long = np.arctan2(y, x)
-        tel["position"] = {
-            "type": "position",
-            # I haven't seen a FITS keyword for reference frame of telescope posiiton
-            "ellipsoid": "GRS80",
-            "units": ["rad", "rad", "m"],
-            "value": np.array([long, lat, r]),
+        tel["direction"] = {
+            "attrs": {
+                "coordinate_system": "geocentric",
+                # I haven't seen a FITS keyword for reference frame of telescope posiiton
+                "frame": "ITRF",
+                "origin_object_name": "earth",
+                "type": "location",
+                "units": "rad",
+            },
+            "data": np.array([long, lat]),
+            "dims": ["ellipsoid_dir_label"],
+            "coords": {
+                "ellipsoid_dir_label": {
+                    "dims": ["ellipsoid_dir_label"],
+                    "data": ["lon", "lat"],
+                }
+            },
+        }
+        tel["distance"] = {
+            "attrs": {
+                "coordinate_system": "geocentric",
+                # I haven't seen a FITS keyword for reference frame of telescope posiiton
+                "frame": "ITRF",
+                "origin_object_name": "earth",
+                "type": "location",
+                "units": "m",
+            },
+            "data": np.array([r]),
+            "dims": ["ellipsoid_dis_label"],
+            "coords": {
+                "ellipsoid_dis_label": {
+                    "dims": ["ellipsoid_dis_label"],
+                    "data": [
+                        "dist",
+                    ],
+                }
+            },
         }
-        helpers["tel_pos"] = tel["position"]
     return tel
 
 
-def _pointing_center_to_metadata(helpers: dict, header) -> dict:
+def _compute_pointing_center(helpers: dict, header) -> dict:
     # Neither helpers or header is modified
     t_axes = helpers["t_axes"]
     long_unit = header[f"CUNIT{t_axes[0]}"]
@@ -256,7 +303,7 @@ def _pointing_center_to_metadata(helpers: dict, header) -> dict:
     pc_lat = float(header[f"CRVAL{t_axes[1]}"]) * unit[1]
     pc_long = pc_long.to(u.rad).value
     pc_lat = pc_lat.to(u.rad).value
-    return {"value": np.array([pc_long, pc_lat]), "initial": True}
+    return make_skycoord_dict([pc_long, pc_lat], units="rad", frame=helpers["ref_sys"])
 
 
 def _user_attrs_from_header(header) -> dict:
@@ -265,6 +312,9 @@ def _user_attrs_from_header(header) -> dict:
         "ALTRPIX",
         "ALTRVAL",
         "BITPIX",
+        "BMAJ",
+        "BMIN",
+        "BPA",
         "BSCALE",
         "BTYPE",
         "BUNIT",
@@ -290,21 +340,20 @@ def _user_attrs_from_header(header) -> dict:
     ]
     regex = r"|".join(
         [
-            "^NAXIS\\d?$",
-            "^CRVAL\\d$",
-            "^CRPIX\\d$",
-            "^CTYPE\\d$",
-            "^CDELT\\d$",
-            "^CUNIT\\d$",
-            "^OBSGEO-(X|Y|Z)$",
-            "^P(C|V)\\d_\\d$",
+            r"^NAXIS\d?$",
+            r"^CRVAL\d$",
+            r"^CRPIX\d$",
+            r"^CTYPE\d$",
+            r"^CDELT\d$",
+            r"^CUNIT\d$",
+            r"^OBSGEO-(X|Y|Z)$",
+            r"^P(C|V)0?\d_0?\d",
         ]
     )
     user = {}
     for k, v in header.items():
-        if re.search(regex, k) or k in exclude:
-            continue
-        user[k.lower()] = v
+        if not (re.search(regex, k) or k in exclude):
+            user[k.lower()] = v
     return user
 
 
@@ -314,9 +363,9 @@ def _beam_attr_from_header(helpers: dict, header) -> Union[dict, str, None]:
     if "BMAJ" in header:
         # single global beam
         beam = {
-            "bmaj": make_quantity(header["BMAJ"], "arcsec"),
-            "bmin": make_quantity(header["BMIN"], "arcsec"),
-            "pa": make_quantity(header["BPA"], "arcsec"),
+            "bmaj": make_quantity(header["BMAJ"], "deg"),
+            "bmin": make_quantity(header["BMIN"], "deg"),
+            "pa": make_quantity(header["BPA"], "deg"),
         }
         return _convert_beam_to_rad(beam)
     elif "CASAMBM" in header and header["CASAMBM"]:
@@ -353,12 +402,41 @@ def _create_dim_map(helpers: dict, header) -> dict:
     return dim_map
 
 
-def _fits_header_to_xds_attrs(hdulist: fits.hdu.hdulist.HDUList) -> dict:
+def _fits_header_to_xds_attrs(
+    hdulist: fits.hdu.hdulist.HDUList, compute_mask: bool
+) -> tuple:
+    # First: Guard for unsupported compressed images
+    for i, hdu in enumerate(hdulist):
+        if isinstance(hdu, fits.CompImageHDU):
+            raise RuntimeError(
+                f"HDU {i}, name={hdu.name} is a CompImageHDU, which is not supported "
+                "for memory-mapping. "
+                "Cannot memory-map compressed FITS image (CompImageHDU). "
+                "Workaround: decompress the FITS using tools like `funpack`, `cfitsio`, "
+                "or Astropy's `.scale()`/`.copy()` workflows"
+            )
     primary = None
+    # FIXME beams is set but never actually used in this function. What's up with that?
     beams = None
     for hdu in hdulist:
         if hdu.name == "PRIMARY":
             primary = hdu
+            # Memory map support check
+            # avoid possibly non-existent hdu.scale_type attribute check and check header instead
+            header = hdu.header
+            scale = hdu.header.get("BSCALE", 1.0)
+            zero = hdu.header.get("BZERO", 0.0)
+            if not (scale == 1.0 and zero == 0.0):
+                raise RuntimeError(
+                    "Cannot memory-map scaled FITS data (BSCALE/BZERO set). "
+                    f"BZERO={zero}, BSCALE={scale}. "
+                    "Workaround: remove scaling with Astropy's"
+                    "  `HDU.data = HDU.data * BSCALE + BZERO` and save a new file"
+                )
+            # NOTE: check for primary.data size being too large removed, since
+            # data is read in chunks, so no danger of exhausting memory
+            # NOTE: sanity-check for ndarray type has been removed to avoid
+            # forcing eager memory load of possibly very large data array.
         elif hdu.name == "BEAMS":
             beams = hdu
         else:
@@ -388,39 +466,72 @@ def _fits_header_to_xds_attrs(hdulist: fits.hdu.hdulist.HDUList) -> dict:
         raise RuntimeError("Could not find both direction axes")
     if dir_axes is not None:
         attrs["direction"] = _xds_direction_attrs_from_header(helpers, header)
-    # FIXME read fits data in chunks in case all data too large to hold in memory
-    has_mask = da.any(da.isnan(primary.data)).compute()
-    attrs["active_mask"] = "mask0" if has_mask else None
-    helpers["has_mask"] = has_mask
+    helpers["has_mask"] = False
+    if compute_mask:
+        # 🧠 Why the primary.data reference here is Safe (does not cause
+        # an eager read of entire data array)
+        # primary.data is a memory-mapped array (because fits.open(..., memmap=True)
+        # is used upstream)
+        # da.from_array(...) wraps this without reading it immediately
+        # The actual read occurs inside:
+        # .map_blocks(...).any().compute()
+        # ...and that triggers blockwise loading via Dask → safe and parallel
+        # 💡 Gotcha
+        # What would be dangerous:
+        # arr = np.isnan(primary.data).any()
+        # That would pull the whole array into memory. But we're not doing that.
+        data_dask = da.from_array(primary.data, chunks="auto")
+        # The following code black has corner case exposure, although the guard should
+        # eliminate it. But there is a cleaner, dask-y way that should work that we implement
+        # next, with cautions
+        # def chunk_has_nan(block):
+        #     if not isinstance(block, np.ndarray) or block.size == 0:
+        #         return False
+        #    return np.isnan(block).any()
+        # helpers["has_mask"] = data_dask.map_blocks(chunk_has_nan, dtype=bool).any().compute()
+        # ✅ Option: np.isnan(data_dask).any().compute()
+        # 🔒 Pros:
+        # Cleaner and shorter (no custom function)
+        # Handles all chunk shapes robustly — no risk of empty inputs
+        # Uses Dask’s own optimized blockwise operations under the hood
+        # ⚠️ Cons:
+        # Might trigger more eager computation if Dask can't optimize well:
+        # If chunks are misaligned or small, Dask might combine many or materialize more blocks than needed
+        # Especially on large images, it could bump memory pressure slightly
+        # But since we already call .compute(), we will load some block data no matter
+        # what — this just changes how much and how smartly.
+        # ✅ Verdict for compute_mask
+        # Because this is explicitly for computing a global has-NaN flag (not building the
+        # dataset), recommend:
+        # helpers["has_mask"] = np.isnan(data_dask).any().compute()
+        # It's concise, robust to shape edge cases, and still parallelized.
+        # We can always revisit it later if perf becomes a concern — and even then,
+        # it's likely a matter of tuning chunks= manually rather than the expression itself.
+        #
+        # This compute will normally be done in parallel
+        helpers["has_mask"] = np.isnan(data_dask).any().compute()
     beam = _beam_attr_from_header(helpers, header)
     if beam != "mb":
-        attrs["beam"] = beam
+        helpers["beam"] = beam
     if "BITPIX" in header:
         v = abs(header["BITPIX"])
+        if v == 16:
+            helpers["dtype"] = "int16"
         if v == 32:
             helpers["dtype"] = "float32"
         elif v == 64:
             helpers["dtype"] = "float64"
         else:
             raise RuntimeError(f'Unhandled data type {header["BITPIX"]}')
-    helpers["btype"] = header["BTYPE"] if "BTYPE" in header else None
-    helpers["bunit"] = header["BUNIT"] if "BUNIT" in header else None
-    attrs["object_name"] = header["OBJECT"] if "OBJECT" in header else None
-    obsdate = {}
-    obsdate["type"] = "time"
-    obsdate["value"] = Time(header["DATE-OBS"], format="isot").mjd
-    obsdate["units"] = "d"
-    obsdate["scale"] = header["TIMESYS"]
-    obsdate["format"] = "MJD"
-    attrs["obsdate"] = obsdate
-    helpers["obsdate"] = obsdate
-    attrs["observer"] = header["OBSERVER"]
-    attrs["pointing_center"] = _pointing_center_to_metadata(helpers, header)
-    attrs["description"] = None
-    attrs["telescope"] = _get_telescope_metadata(helpers, header)
+    helpers["obsdate"] = make_time_measure_dict(
+        data=Time(header["DATE-OBS"], format="isot").mjd,
+        units=["d"],
+        scale=header["TIMESYS"],
+        time_format="MJD",
+    )
+
     # TODO complete _make_history_xds when spec has been finalized
     # attrs['history'] = _make_history_xds(header)
-    attrs["user"] = _user_attrs_from_header(header)
     return attrs, helpers, header
 
 
@@ -453,8 +564,8 @@ def _create_coords(
     helpers["sphr_dims"] = sphr_dims
     coords = {}
     coords["time"] = _get_time_values(helpers)
-    coords["polarization"] = _get_pol_values(helpers)
     coords["frequency"] = _get_freq_values(helpers)
+    coords["polarization"] = _get_pol_values(helpers)
     coords["velocity"] = (["frequency"], _get_velocity_values(helpers))
     if len(sphr_dims) > 0:
         for i, c in enumerate(["l", "m"]):
@@ -487,8 +598,8 @@ def _create_coords(
                 cdelt=pick(helpers["cdelt"]),
                 cunit=pick(helpers["cunit"]),
             )
+            helpers["cunit"] = my_ret["units"]
             for j, i in enumerate(dir_axes):
-                helpers["cunit"][i] = my_ret["unit"][j]
                 helpers["crval"][i] = my_ret["ref_val"][j]
                 helpers["cdelt"][i] = my_ret["inc"][j]
             coords[my_ret["axis_name"][0]] = (["l", "m"], my_ret["value"][0])
@@ -497,41 +608,17 @@ def _create_coords(
     else:
         # Fourier image
         coords["u"], coords["v"] = _get_uv_values(helpers)
+    coords["beam_param"] = ["major", "minor", "pa"]
     xds = xr.Dataset(coords=coords)
     return xds
 
 
 def _get_time_values(helpers):
-    return [helpers["obsdate"]["value"]]
+    return [helpers["obsdate"]["data"]]
 
 
 def _get_pol_values(helpers):
-    # as mapped in casacore Stokes.h
-    stokes_map = [
-        "Undefined",
-        "I",
-        "Q",
-        "U",
-        "V",
-        "RR",
-        "RL",
-        "LR",
-        "LL",
-        "XX",
-        "XY",
-        "YX",
-        "YY",
-        "RX",
-        "RY",
-        "LX",
-        "LY",
-        "XR",
-        "XL",
-        "YR",
-        "YL",
-        "PP",
-        "PQ",
-    ]
+
     idx = helpers["ctype"].index("STOKES")
     if idx >= 0:
         vals = []
@@ -541,7 +628,13 @@ def _get_pol_values(helpers):
         stokes_start_idx = crval - cdelt * crpix
         for i in range(helpers["shape"][idx]):
             stokes_idx = (stokes_start_idx + i) * cdelt
-            vals.append(stokes_map[stokes_idx])
+            if 0 <= stokes_idx < len(stokes_types):
+                # stokes_types provides the index-label mapping from casacore Stokes.h
+                vals.append(stokes_types[stokes_idx])
+            else:
+                raise RuntimeError(
+                    "Can't find the Stokes type using the FITS header index"
+                )
         return vals
     else:
         return ["I"]
@@ -581,9 +674,9 @@ def _get_freq_values(helpers: dict) -> list:
         freq, vel = _freq_from_vel(
             crval, cdelt, crpix, cunit, "Z", helpers["shape"][v_idx], restfreq
         )
-        helpers["velocity"] = vel["value"] * u.Unit(vel["unit"])
-        helpers["crval"][v_idx] = (freq["crval"] * u.Unit(freq["unit"])).to(u.Hz).value
-        helpers["cdelt"][v_idx] = (freq["cdelt"] * u.Unit(freq["unit"])).to(u.Hz).value
+        helpers["velocity"] = vel["value"] * u.Unit(vel["units"])
+        helpers["crval"][v_idx] = (freq["crval"] * u.Unit(freq["units"])).to(u.Hz).value
+        helpers["cdelt"][v_idx] = (freq["cdelt"] * u.Unit(freq["units"])).to(u.Hz).value
         return list(freq["value"])
     else:
         return [1420e6]
@@ -602,6 +695,9 @@ def _get_velocity_values(helpers: dict) -> list:
         return v
 
 
+# FIXME change namee, even if there is only a single beam, we make a
+# multi beam array using it. If we have a beam, it will always be
+# "mutltibeam" is name is redundant and confusing
 def _do_multibeam(xds: xr.Dataset, imname: str) -> xr.Dataset:
     """Only run if we are sure there are multiple beams"""
     hdulist = fits.open(imname)
@@ -615,28 +711,43 @@ def _do_multibeam(xds: xr.Dataset, imname: str) -> xr.Dataset:
             )
             nchan = header["NCHAN"]
             npol = header["NPOL"]
-            beam_array = np.zeros([1, npol, nchan, 3])
+            beam_array = np.zeros([1, nchan, npol, 3])
             data = hdu.data
+            hdulist.close()
             for t in data:
-                beam_array[0, t[4], t[3]] = t[0:3]
+                beam_array[0, t[3], t[4]] = t[0:3]
             for i in (0, 1, 2):
                 beam_array[:, :, :, i] = (
                     (beam_array[:, :, :, i] * units[i]).to("rad").value
                 )
-            xdb = xr.DataArray(
-                beam_array, dims=["time", "polarization", "frequency", "beam_param"]
-            )
-            xdb = xdb.rename("beam")
-            xdb = xdb.assign_coords(beam_param=["major", "minor", "pa"])
-            xdb.attrs["units"] = "rad"
-            xds["beam"] = xdb
-            return xds
+            return _create_beam_data_var(xds, beam_array)
     raise RuntimeError(
         "It looks like there should be a BEAMS table but no "
         "such table found in FITS file"
     )
 
 
+def _add_beam(xds: xr.Dataset, helpers: dict) -> xr.Dataset:
+    nchan = xds.sizes["frequency"]
+    npol = xds.sizes["polarization"]
+    beam_array = np.zeros([1, nchan, npol, 3])
+    beam_array[0, :, :, 0] = helpers["beam"]["bmaj"]["data"]
+    beam_array[0, :, :, 1] = helpers["beam"]["bmin"]["data"]
+    beam_array[0, :, :, 2] = helpers["beam"]["pa"]["data"]
+    return _create_beam_data_var(xds, beam_array)
+
+
+def _create_beam_data_var(xds: xr.Dataset, beam_array: np.array) -> xr.Dataset:
+    xdb = xr.DataArray(
+        beam_array, dims=["time", "frequency", "polarization", "beam_param"]
+    )
+    xdb = xdb.rename("BEAM")
+    xdb = xdb.assign_coords(beam_param=["major", "minor", "pa"])
+    xdb.attrs["units"] = "rad"
+    xds["BEAM"] = xdb
+    return xds
+
+
 def _get_uv_values(helpers: dict) -> tuple:
     shape = helpers["shape"]
     ctype = helpers["ctype"]
@@ -665,14 +776,22 @@ def _add_sky_or_aperture(
     xds: xr.Dataset,
     ary: Union[np.ndarray, da.array],
     dim_order: list,
+    header,
     helpers: dict,
     has_sph_dims: bool,
 ) -> xr.Dataset:
     xda = xr.DataArray(ary, dims=dim_order)
-    image_type = helpers["btype"]
-    unit = helpers["bunit"]
-    xda.attrs[_image_type] = image_type
-    xda.attrs["units"] = unit
+    for h, a in zip(
+        ["BUNIT", "BTYPE", "OBJECT", "OBSERVER"],
+        ["units", _image_type, "object_name", "observer"],
+    ):
+        if h in header:
+            xda.attrs[a] = header[h]
+    xda.attrs["obsdate"] = helpers["obsdate"].copy()
+    xda.attrs["pointing_center"] = _compute_pointing_center(helpers, header)
+    xda.attrs["telescope"] = _get_telescope_metadata(helpers, header)
+    xda.attrs["description"] = None
+    xda.attrs["user"] = _user_attrs_from_header(header)
     name = "SKY" if has_sph_dims else "APERTURE"
     xda = xda.rename(name)
     xds[xda.name] = xda
@@ -680,8 +799,11 @@ def _add_sky_or_aperture(
         pp = da if type(xda[0].data) == dask.array.core.Array else np
         mask = pp.isnan(xda)
         mask.attrs = {}
-        mask = mask.rename("mask0")
-        xds["mask0"] = mask
+        mask = mask.rename("MASK0")
+        xds["MASK0"] = mask
+        xda.attrs["active_mask"] = "MASK0"
+    xda = xda.rename(name)
+    xds[xda.name] = xda
     return xds
 
 
@@ -790,10 +912,10 @@ def _get_transpose_list(helpers: dict) -> tuple:
             or b.startswith("vopt")
             or b.startswith("vrad")
         ):
-            transpose_list[2] = i
+            transpose_list[1] = i
             not_covered.remove("f")
         elif b.startswith("stok"):
-            transpose_list[1] = i
+            transpose_list[2] = i
             not_covered.remove("s")
         else:
             raise RuntimeError(f"Unhandled axis name {c}")
@@ -810,12 +932,12 @@ def _get_transpose_list(helpers: dict) -> tuple:
 
 def _read_image_chunk(img_full_path, shapes: tuple, starts: tuple) -> np.ndarray:
     hdulist = fits.open(img_full_path, memmap=True)
-    s = []
-    for start, length in zip(starts, shapes):
-        s.append(slice(start, start + length))
-    t = tuple(s)
-    z = hdulist[0].data[t]
+    hdu = hdulist[0]
+    # Chunk slice
+    slices = tuple(
+        slice(start, start + length) for start, length in zip(starts, shapes)
+    )
+    chunk = hdu.data[slices]
     hdulist.close()
-    # delete to avoid having a reference to a mem-mapped hdulist
     del hdulist
-    return z
+    return chunk
