@@ -4,14 +4,19 @@ from pathlib import Path
 import re
 from typing import Any, Callable, Dict, List, Tuple, Union
 
+import dask.array as da
 import numpy as np
 import pandas as pd
 import xarray as xr
 
 import astropy.units
-from casacore import tables
 
-from .table_query import open_query, open_table_ro
+try:
+    from casacore import tables
+except ImportError:
+    import xradio._utils._casacore.casacore_from_casatools as tables
+
+from .table_query import open_query, open_table_ro, TableManager
 from xradio._utils.list_and_array import get_pad_value
 
 CASACORE_TO_PD_TIME_CORRECTION = 3_506_716_800.0
@@ -41,16 +46,20 @@ def convert_casacore_time(
     rawtimes: np.ndarray, convert_to_datetime: bool = True
 ) -> np.ndarray:
     """
-    Read time columns to datetime format
-    pandas datetimes are referenced against a 0 of 1970-01-01
-    CASA's modified julian day reference time is (of course) 1858-11-17
+    Convert data from casacore time columns to a different format, either:
+    a) pandas style datetime,
+    b) simply seconds from 1970-01-01 00:00:00 UTC (as used in the Unix scale of
+       astropy).
+
+    Pandas datetimes and Unix times are referenced against a 0 of 1970-01-01.
+    CASA's (casacore) modified julian day reference time is (of course) 1858-11-17.
 
     This requires a correction of 3506716800 seconds which is hardcoded to save time
 
     Parameters
     ----------
     rawtimes : np.ndarray
-        times in casacore ref
+        time values wrt casacore reference
     convert_to_datetime : bool (Default value = True)
         whether to produce pandas style datetime
 
@@ -307,6 +316,8 @@ def add_units_measures(
                 ):  # Little fix for Meerkat data where the units are a string.
                     cc_units = [cc_units]
 
+                if isinstance(cc_units, np.ndarray):
+                    cc_units = cc_units.tolist()
                 if not isinstance(cc_units, list) or not cc_units:
                     logger.warning(
                         f"Invalid units found for column/variable {col}: {cc_units}"
@@ -344,70 +355,6 @@ def add_units_measures(
     return mvars
 
 
-def make_freq_attrs(spw_xds: xr.Dataset, spw_id: int) -> Dict[str, Any]:
-    """
-    Grab the units/measure metainfo for the xds.freq dimension of a
-    parttion from the SPECTRAL_WINDOW subtable CTDS attributes.
-
-    Has to read xds_spw.meas_freq_ref and use it as index in the CTDS
-    'VarRefCol' attrs of CHAN_FREQ and REF_FREQUENCY to give a
-    reference frame to xds_spw.ref_frequency and xds_spw.chan_freq
-    (then the ref frame from the second will be pulled to
-    xds.freq.attrs)
-
-    Parameters
-    ----------
-    spw_xds : xr.Dataset
-        (metainfo) SPECTRAL_WINDOW xds
-    spw_id : int
-        SPW id of a partition
-
-    Returns
-    -------
-    Dict[str, Any]
-        attributes (units/measure) for the freq dim of a partition
-    """
-    fallback_TabRefTypes = [
-        "REST",
-        "LSRK",
-        "LSRD",
-        "BARY",
-        "GEO",
-        "TOPO",
-        "GALACTO",
-        "LGROUP",
-        "CMB",
-    ]
-
-    ctds_cols = spw_xds.attrs["other"]["msv2"]["ctds_attrs"]["column_descriptions"]
-    cfreq = ctds_cols["CHAN_FREQ"]
-
-    cf_attrs = spw_xds.data_vars["CHAN_FREQ"].attrs
-    if "MEASINFO" in cfreq["keywords"] and "VarRefCol" in cfreq["keywords"]["MEASINFO"]:
-        fattrs = cfreq["keywords"]["MEASINFO"]
-        var_ref_col = fattrs["VarRefCol"]
-        # This should point to the SPW/MEAS_FREQ_REF col
-        meas_freq_ref_idx = spw_xds.data_vars[var_ref_col].values[spw_id]
-
-        if "TabRefCodes" not in fattrs or "TabRefTypes" not in fattrs:
-            # Datasets like vla/ic2233_1.ms say "VarRefCol" but "TabRefTypes" is missing
-            ref_frame = fallback_TabRefTypes[meas_freq_ref_idx]
-        else:
-            ref_type_code = fattrs["TabRefCodes"][meas_freq_ref_idx]
-            ref_frame = fattrs["TabRefTypes"][ref_type_code]
-
-        cf_attrs["measure"] = {
-            "type": fattrs["type"],
-            "ref_frame": ref_frame,
-        }
-
-        # Also set the 'VarRefCol' for CHAN_FREQ and REF_FREQUENCEY
-        spw_xds.data_vars["CHAN_FREQ"].attrs.update(cf_attrs)
-        spw_xds.data_vars["REF_FREQUENCY"].attrs.update(cf_attrs)
-
-    return cf_attrs
-
-
 def redimension_ms_subtable(xds: xr.Dataset, subt_name: str) -> xr.Dataset:
     """
     Expand a MeasurementSet subtable xds from single dimension (row)
@@ -544,8 +491,8 @@ def load_generic_table(
     tname : str
         (sub)table name, for example 'SOURCE' for myms.ms/SOURCE
     timecols : Union[List[str], None] (Default value = None)
-        column names to convert to numpy datetime format.
-        leaves times as their original casacore format.
+        Names of time column(s), to convert from casacore times to 1970-01-01 scale
+        An empty list leaves times as their original casacore format.
     ignore : Union[List[str], None] (Default value = None)
         list of column names to ignore and not try to read.
     rename_ids : Dict[str, str] (Default value = None)
@@ -741,7 +688,7 @@ def load_generic_cols(
     tb_tool : tables.table
         table to load the columns
     timecols : Union[List[str], None]
-        columns names to convert to datetime format
+        column names to convert from casacore time format
     ignore : Union[List[str], None]
         list of column names to skip and not try to load.
 
@@ -821,7 +768,7 @@ def load_fixed_size_cols(
     tb_tool : tables.table
         table to red the columns
     timecols : Union[List[str], None]
-        columns names to convert to datetime format
+        column names to convert from casacore time format
     ignore : Union[List[str], None]
         list of column names to skip and not try to load.
 
@@ -916,7 +863,8 @@ def raw_col_data_to_coords_vars(
     data: np.ndarray :
         column data
     timecols: Union[List[str], None]
-        columns to be treated as TIME-related
+        columns to be treated as TIME-related (they are coordinate, need conversion from
+        casacore time format.
 
     Returns
     -------
@@ -946,7 +894,7 @@ def raw_col_data_to_coords_vars(
             data = convert_mjd_time(data).astype("float64") / 1e9
         else:
             try:
-                data = convert_casacore_time(data)
+                data = convert_casacore_time(data, False)
             except pd.errors.OutOfBoundsDatetime as exc:
                 if inpath.endswith("WEATHER"):
                     # intentionally not callling logging.exception
@@ -986,7 +934,7 @@ def raw_col_data_to_coords_vars(
 
 def get_pad_value_in_tablerow_column(trows: tables.tablerow, col: str) -> object:
     """
-    Gets the pad value for the type of a column (IMPORTANTLY) as froun in the
+    Gets the pad value for the type of a column (IMPORTANTLY) as found in the
     the type specified in the row / column value dict returned by tablerow.
     This can differ from the type of the column as given in the casacore
     column descriptions. See https://github.com/casangi/xradio/issues/242.
@@ -1188,7 +1136,7 @@ def read_col_chunk(
     np.ndarray
     """
     # TODO: consider calling load_col_chunk() from inside the withs
-    # for read_delayed_pointing_table and read_expanded_main_table
+    # for read_expanded_main_table
     with open_table_ro(infile) as mtable:
         with open_query(mtable, ts_taql) as query:
             if (len(cshape) == 2) or (col == "UVW"):  # all the scalars and UVW
@@ -1207,18 +1155,114 @@ def read_col_chunk(
     return fulldata
 
 
-def read_col_conversion(
-    tb_tool: tables.table,
+def read_col_conversion_numpy(
+    table_manager: TableManager,
     col: str,
     cshape: Tuple[int],
     tidxs: np.ndarray,
     bidxs: np.ndarray,
     use_table_iter: bool,
+    time_chunksize: int,
 ) -> np.ndarray:
     """
     Function to perform delayed reads from table columns when converting
     (no need for didxs)
 
+    Parameters
+    ----------
+    table_manager : TableManager
+
+    col : str
+
+    cshape : Tuple[int]
+
+    tidxs : np.ndarray
+
+    bidxs : np.ndarray
+
+    use_table_iter : bool
+
+    Returns
+    -------
+    np.ndarray
+    """
+
+    # Workaround for https://github.com/casacore/python-casacore/issues/130
+    # WARNING: Assumes tb_tool is a single measurement set not an MMS.
+    # WARNING: Assumes the num_frequencies * num_polarizations < 2**29. If false,
+    # https://github.com/casacore/python-casacore/issues/130 isn't mitigated.
+
+    with table_manager.get_table() as tb_tool:
+
+        # Use casacore to get the shape of a row for this column
+        #################################################################################
+
+        # getcolshapestring() only works for array-valued columns.
+        # For scalar columns (e.g., EXPOSURE, TIME_CENTROID), it raises a RuntimeError.
+        # So we first check if the column is scalar to avoid that.
+        if tb_tool.isscalarcol(col):
+            extra_dimensions = ()
+        else:
+            # Get the shape string for the first row of the column (e.g., "[4, 2]")
+            shape_string = tb_tool.getcolshapestring(col)[0]
+
+            # Convert the shape string into a tuple of integers (e.g., (4, 2)) that numpy
+            # understands.
+            extra_dimensions = tuple(
+                int(dim) for dim in shape_string.strip("[]").split(", ")
+            )
+
+        #################################################################################
+
+        # Get dtype of the column. Only read first row from disk
+        col_dtype = np.array(tb_tool.col(col)[0]).dtype
+        # Use a custom/safe fill value (https://github.com/casangi/xradio/issues/219)
+        fill_value = get_pad_value(col_dtype)
+
+        # Construct a numpy array to populate. `data` has shape (n_times, n_baselines, n_frequencies, n_polarizations)
+        data = np.full(cshape + extra_dimensions, fill_value, dtype=col_dtype)
+
+        # Use built-in casacore table iterator to populate the data column by unique times.
+        if use_table_iter:
+            start_row = 0
+            for ts in tb_tool.iter("TIME", sort=False):
+                num_rows = ts.nrows()
+
+                # Create small temporary array to store the partial column
+                tmp_arr = np.full(
+                    (num_rows,) + extra_dimensions, fill_value, dtype=col_dtype
+                )
+
+                # Note we don't use `getcol()` because it's less safe. See:
+                # https://github.com/casacore/python-casacore/issues/130#issuecomment-463202373
+                ts.getcolnp(col, tmp_arr)
+
+                # Get the slice of rows contained in `tmp_arr`.
+                # Used to get the relevant integer indexes from `tidxs` and `bidxs`
+                tmp_slice = slice(start_row, start_row + num_rows)
+
+                # Copy `tmp_arr` into correct elements of `tmp_arr`
+                data[tidxs[tmp_slice], bidxs[tmp_slice]] = tmp_arr
+                start_row += num_rows
+        else:
+            data[tidxs, bidxs] = tb_tool.getcol(col)
+
+    return data
+
+
+def read_col_conversion_dask(
+    table_manager: TableManager,
+    col: str,
+    cshape: Tuple[int],
+    tidxs: np.ndarray,
+    bidxs: np.ndarray,
+    use_table_iter: bool,
+    time_chunksize: int,
+) -> da.Array:
+    """
+    Function to perform delayed reads from table columns when converting
+    (no need for didxs)
+
     Parameters
     ----------
     tb_tool : tables.table
@@ -1233,68 +1277,110 @@ def read_col_conversion(
 
     Returns
     -------
-    np.ndarray
+    da.Array
     """
 
-    # Workaround for https://github.com/casacore/python-casacore/issues/130
-    # WARNING: Assumes tb_tool is a single measurement set not an MMS.
-    # WARNING: Assumes the num_frequencies * num_polarizations < 2**29. If false,
-    # https://github.com/casacore/python-casacore/issues/130 isn't mitigated.
-
     # Use casacore to get the shape of a row for this column
     #################################################################################
 
-    # Get the total number of rows in the base measurement set
-    nrows_total = tb_tool.nrows()
+    with table_manager.get_table() as tb_tool:
+        first_row = tb_tool.row(col)[0][col]
 
-    # getcolshapestring() only works on columns where a row element is an
-    # array ie. fails for TIME
-    # Assumes the RuntimeError is because the column is a scalar
-    try:
-        shape_string = tb_tool.getcolshapestring(col)[0]
-        # Convert `shape_string` into a tuple that numpy understands
-        extra_dimensions = tuple(
-            [
-                int(idx)
-                for idx in shape_string.replace("[", "").replace("]", "").split(", ")
-            ]
-        )
-    except RuntimeError:
+    if isinstance(first_row, np.ndarray):
+        extra_dimensions = first_row.shape
+
+    else:
         extra_dimensions = ()
 
+    # Use dask primitives to lazily read chunks of data from the MeasurementSet
+    # Takes inspiration from dask_image https://image.dask.org/en/latest/
     #################################################################################
 
-    # Get dtype of the column. Only read first row from disk
-    col_dtype = np.array(tb_tool.col(col)[0]).dtype
-    # Use a custom/safe fill value (https://github.com/casangi/xradio/issues/219)
-    fill_value = get_pad_value(col_dtype)
+    # Get dtype of the column. Wrap in numpy array in case of scalar column
+    col_dtype = np.array(first_row).dtype
 
-    # Construct a numpy array to populate. `data` has shape (n_times, n_baselines, n_frequencies, n_polarizations)
-    data = np.full(cshape + extra_dimensions, fill_value, dtype=col_dtype)
+    # Get the number of rows for a single TIME value
+    num_utimes = cshape[0]
+    rows_per_time = cshape[1]
 
-    # Use built-in casacore table iterator to populate the data column by unique times.
-    if use_table_iter:
-        start_row = 0
-        for ts in tb_tool.iter("TIME", sort=False):
-            num_rows = ts.nrows()
+    # Calculate the chunks of unique times that gives the target chunk sizes
+    tmp_chunks = da.core.normalize_chunks(time_chunksize, (num_utimes,))[0]
 
-            # Create small temporary array to store the partial column
-            tmp_arr = np.full(
-                (num_rows,) + extra_dimensions, fill_value, dtype=col_dtype
-            )
+    sum = 0
+    arr_start_end_rows = []
+    for chunk in tmp_chunks:
+        start = (sum) * rows_per_time
+        end = (sum + chunk) * rows_per_time
 
-            # Note we don't use `getcol()` because it's less safe. See:
-            # https://github.com/casacore/python-casacore/issues/130#issuecomment-463202373
-            ts.getcolnp(col, tmp_arr)
+        arr_start_end_rows.append((start, end))
+        sum += chunk
 
-            # Get the slice of rows contained in `tmp_arr`.
-            # Used to get the relevant integer indexes from `tidxs` and `bidxs`
-            tmp_slice = slice(start_row, start_row + num_rows)
+    # Store the start and end rows that should be read for the chunk
+    arr_start_end_rows = da.from_array(arr_start_end_rows, chunks=(1, 2))
 
-            # Copy `tmp_arr` into correct elements of `tmp_arr`
-            data[tidxs[tmp_slice], bidxs[tmp_slice]] = tmp_arr
-            start_row += num_rows
-    else:
-        data[tidxs, bidxs] = tb_tool.getcol(col)
+    # Specify the output shape `load_col_chunk`
+    output_chunkshape = (tmp_chunks, cshape[1]) + extra_dimensions
+
+    # Apply `load_col_chunk` to each chunk
+    data = arr_start_end_rows.map_blocks(
+        load_col_chunk,
+        table_manager=table_manager,
+        col_name=col,
+        col_dtype=col_dtype,
+        tidxs=tidxs,
+        bidxs=bidxs,
+        rows_per_time=rows_per_time,
+        cshape=cshape,
+        extra_dimensions=extra_dimensions,
+        drop_axis=[1],
+        new_axis=list(range(1, len(cshape + extra_dimensions))),
+        meta=np.array([], dtype=col_dtype),
+        chunks=output_chunkshape,
+    )
 
     return data
+
+
+def load_col_chunk(
+    x,
+    table_manager,
+    col_name,
+    col_dtype,
+    tidxs,
+    bidxs,
+    rows_per_time,
+    cshape,
+    extra_dimensions,
+):
+    start_row = x[0][0]
+    end_row = x[0][1]
+    num_rows = end_row - start_row
+    assert (num_rows % rows_per_time) == 0
+    num_utimes = num_rows // rows_per_time
+
+    # Create memory buffer to populate with data from disk
+    row_data = np.full((num_rows,) + extra_dimensions, np.nan, dtype=col_dtype)
+
+    # Load data from the column
+    # Release the casacore table as soon as possible
+    with table_manager.get_table() as tb_tool:
+        tb_tool.getcolnp(col_name, row_data, startrow=start_row, nrow=num_rows)
+
+    # Initialise reshaped numpy array
+    reshaped_data = np.full(
+        (num_utimes, cshape[1]) + extra_dimensions, np.nan, dtype=col_dtype
+    )
+
+    # Create slice object for readability
+    slc = slice(start_row, end_row)
+    tidxs_slc = tidxs[slc]
+
+    tidxs_slc = (
+        tidxs_slc - tidxs_slc[0]
+    )  # Indices of reshaped_data along time differ from values in tidxs. Assumes first time is earliest time
+    bidxs_slc = bidxs[slc]
+
+    # Populate `reshaped_data` with `row_data`
+    reshaped_data[tidxs_slc, bidxs_slc] = row_data
+
+    return reshaped_data
