@@ -1,18 +1,26 @@
 import toolviper.utils.logger as logger
 import os
 import time
-from typing import Tuple, Union
+from typing import Optional, Tuple, Union
 
 import numpy as np
 import xarray as xr
+from numpy.typing import ArrayLike
 
 from xradio._utils.coord_math import convert_to_si_units
+from xradio._utils.dict_helpers import (
+    make_time_measure_attrs,
+    make_spectral_coord_measure_attrs,
+)
 from xradio._utils.schema import (
     column_description_casacore_to_msv4_measure,
     convert_generic_xds_to_xradio_schema,
 )
-from .subtables import subt_rename_ids
-from ._tables.read import (
+from xradio.measurement_set._utils._utils.interpolate import (
+    interpolate_to_time,
+)
+from xradio.measurement_set._utils._msv2.subtables import subt_rename_ids
+from xradio.measurement_set._utils._msv2._tables.read import (
     load_generic_table,
     make_taql_where_between_min_max,
     table_exists,
@@ -20,12 +28,7 @@ from ._tables.read import (
 )
 
 
-standard_time_coord_attrs = {
-    "type": "time",
-    "units": ["s"],
-    "scale": "utc",
-    "format": "unix",
-}
+standard_time_coord_attrs = make_time_measure_attrs(time_format="unix")
 
 
 def rename_and_interpolate_to_time(
@@ -88,57 +91,6 @@ def rename_and_interpolate_to_time(
     return renamed_time_xds
 
 
-def interpolate_to_time(
-    xds: xr.Dataset,
-    interp_time: Union[xr.DataArray, None],
-    message_prefix: str,
-    time_name: str = "time",
-) -> xr.Dataset:
-    """
-    Interpolate the time coordinate of the input xarray dataset to the
-    a data array. This can be used for example to interpolate a pointing_xds
-    to the time coord of the (main) MSv4, or similarly the ephemeris
-    data variables of a field_and_source_xds.
-
-    Uses interpolation method "linear", unless the source number of points is
-    1 in which case "nearest" is used, to avoid divide-by-zero issues.
-
-    Parameters:
-    ----------
-    xds : xr.Dataset
-        Xarray dataset to interpolate (presumably a pointing_xds or an xds of
-        ephemeris variables)
-    interp_time : Union[xr.DataArray, None]
-        Time axis to interpolate the dataset to (usually main MSv4 time)
-    message_prefix: str
-        A prefix for info/debug/etc. messages
-
-    Returns:
-    -------
-    interpolated_xds : xr.Dataset
-        xarray dataset with time axis interpolated to interp_time.
-    """
-    if interp_time is not None:
-        points_before = xds[time_name].size
-        if points_before > 1:
-            method = "linear"
-        else:
-            method = "nearest"
-        xds = xds.interp(
-            {time_name: interp_time.data}, method=method, assume_sorted=True
-        )
-        # scan_name sneaks in as a coordinate of the main time axis, drop it
-        if "scan_name" in xds.coords:
-            xds = xds.drop_vars("scan_name")
-        points_after = xds[time_name].size
-        logger.debug(
-            f"{message_prefix}: interpolating the time coordinate "
-            f"from {points_before} to {points_after} points"
-        )
-
-    return xds
-
-
 def make_taql_where_weather(
     in_file: str, ant_xds_station_name_ids: xr.DataArray
 ) -> str:
@@ -172,7 +124,131 @@ def make_taql_where_weather(
     return taql_where
 
 
-def create_weather_xds(in_file: str, ant_xds_station_name_ids: xr.DataArray):
+def prepare_generic_weather_xds_and_station_name(
+    generic_weather_xds: xr.Dataset,
+    in_file: str,
+    ant_position_with_ids: xr.DataArray,
+    has_asdm_station_position: bool,
+) -> tuple[xr.Dataset, np.ndarray]:
+    """
+    A generic_weather_xds loaded with load_generic_table() might still need to be reloaded
+    with an additional WHERE condition to constrain the indices of antennas. But this depends on whether
+    ASDM/importasdm extension columns are present or not.
+
+    This also prepares the station_name values:
+    - if has_asdm_station_ids:
+       - tries to find from ASDM_STATION the station names,
+       - otherwise, takes ids (antenna_ids in generic_weather were actually the ASDM_STATION_IDs
+    - else: get the values from antenna_xds (the stations present)
+
+
+    Parameters
+    ----------
+    generic_weather_xds : xr.Dataset
+        generic dataset read from an MSv2 WEATHER subtable
+    in_file : str
+        Input MS name.
+    ant_position_with_ids : xr.DataArray
+        antenna_position data var from the antenna_xds (expected to still include the initial ANTENNA_ID
+        coordinate as well as other coordinates from the antenna_xds)
+    has_asdm_station_position : bool
+        wHether this generic weatehr_xds should be treated as including the nonstandard extensions
+        NS_WX_STATION_ID and NS_WX_STATION_POSITION as created by CASA/importasdm (ALMA and VLA).
+
+    Returns
+    -------
+    (generic_weather_xds, station_name): tuple[[xarray.Dataset, numpy.ndarray]
+        Weather Xarray Dataset prepared for generic conversion to MSv4, values for the station_name coordinate
+    """
+
+    if has_asdm_station_position:
+        asdm_station_path = os.path.join(in_file, "ASDM_STATION")
+        if table_exists(asdm_station_path):
+            asdm_station_xds = load_generic_table(in_file, "ASDM_STATION")
+            station_name = asdm_station_xds.name.values[
+                generic_weather_xds["ANTENNA_ID"].values
+            ]
+        else:
+            # if no info from ASDM_STATION, use the indices from antenna_id which was actually the NS_WX_STATION_ID
+            len_antenna_id = generic_weather_xds.sizes["ANTENNA_ID"]
+            station_name = list(
+                map(
+                    lambda x, y: x + "_" + y,
+                    ["Station"] * len_antenna_id,
+                    generic_weather_xds["ANTENNA_ID"].values.astype(str),
+                )
+            )
+
+    else:
+        taql_where = make_taql_where_weather(in_file, ant_position_with_ids)
+        generic_weather_xds = load_generic_table(
+            in_file,
+            "WEATHER",
+            timecols=["TIME"],
+            rename_ids=subt_rename_ids["WEATHER"],
+            taql_where=taql_where,
+        )
+
+        if not generic_weather_xds.data_vars:
+            # for example when the weather subtable only has info for antennas/stations
+            # not present in the MSv4 (no overlap between antennas loaded in ant_xds and weather)
+            return None, None
+
+        stations_present = ant_position_with_ids.sel(
+            antenna_id=generic_weather_xds["ANTENNA_ID"]
+        ).station_name
+        station_name = stations_present.values
+
+    return generic_weather_xds, station_name
+
+
+def finalize_station_position(
+    weather_xds: xr.Dataset, ant_position_with_ids, has_asdm_station_position: bool
+) -> xr.Dataset:
+    """
+    For a STATION_POSITION data var being added to a weather_xds, make sure coordinates and dimensions
+    are conforming to the schema.
+
+    Parameters
+    ----------
+    weather_xds : xr.Dataset
+        weather_xds where we still need to ensure the right coordinates and attributes
+    ant_position_with_ids : xr.DataArray
+        antenna_position data var from the antenna_xds (expected to still include the initial ANTENNA_ID
+        coordinate as well as other coordinates from the antenna_xds)
+    has_asdm_station_position : bool
+        Whether this generic weatehr_xds should be treated as including the nonstandard extensions
+        NS_WX_STATION_ID and NS_WX_STATION_POSITION as created by CASA/importasdm (ALMA and VLA).
+
+    Returns
+    -------
+    weather_xds: xarray.Dataset
+        Weather Xarray Dataset with all coordinates and attributes in STATION_POSITION
+    """
+    if has_asdm_station_position:
+        # STATION_POSITION has been created but needs prooper dimensions and attrs
+        # Drop the time dim
+        weather_xds["STATION_POSITION"] = weather_xds["STATION_POSITION"].sel(
+            time_weather=0, drop=True, method="nearest"
+        )
+        # borrow location frame attributes from antenna position
+        weather_xds["STATION_POSITION"].attrs = ant_position_with_ids.attrs
+    else:
+        # borrow from ant_position_with_ids but without carrying over other coords
+        weather_xds = weather_xds.assign(
+            {
+                "STATION_POSITION": (
+                    ["station_name", "cartesian_pos_label"],
+                    ant_position_with_ids.values,
+                    ant_position_with_ids.attrs,
+                )
+            }
+        )
+
+    return weather_xds
+
+
+def create_weather_xds(in_file: str, ant_position_with_ids: xr.DataArray):
     """
     Creates a Weather Xarray Dataset from a MS v2 WEATHER table.
 
@@ -180,8 +256,9 @@ def create_weather_xds(in_file: str, ant_xds_station_name_ids: xr.DataArray):
     ----------
     in_file : str
         Input MS name.
-    ant_xds_station_name_ids : xr.DataArray
-        station name data array from antenna_xds, with name/id information
+    ant_position_with_ids : xr.DataArray
+        antenna_position data var from the antenna_xds (expected to still including the initial ANTENNA_ID coordinate
+        as wellas other coordinates from the antenna_xds)
 
     Returns
     -------
@@ -190,34 +267,35 @@ def create_weather_xds(in_file: str, ant_xds_station_name_ids: xr.DataArray):
     """
 
     try:
-        taql_where = make_taql_where_weather(in_file, ant_xds_station_name_ids)
         generic_weather_xds = load_generic_table(
             in_file,
             "WEATHER",
+            timecols=["TIME"],
             rename_ids=subt_rename_ids["WEATHER"],
-            taql_where=taql_where,
         )
     except ValueError as _exc:
         return None
 
-    if not generic_weather_xds.data_vars:
-        # for example when the weather subtable only has info for antennas/stations
-        # not present in the MSv4 (no overlap between antennas loaded in ant_xds and weather)
+    has_asdm_station_position = (
+        "NS_WX_STATION_POSITION" in generic_weather_xds.data_vars
+    )
+    generic_weather_xds, station_name = prepare_generic_weather_xds_and_station_name(
+        generic_weather_xds, in_file, ant_position_with_ids, has_asdm_station_position
+    )
+    if not generic_weather_xds:
         return None
 
     weather_xds = xr.Dataset(attrs={"type": "weather"})
-    stations_present = ant_xds_station_name_ids.sel(
-        antenna_id=generic_weather_xds["ANTENNA_ID"]
-    )
     coords = {
-        "station_name": stations_present.data,
-        "antenna_name": stations_present.coords["antenna_name"].data,
+        "station_name": station_name,
+        "cartesian_pos_label": ["x", "y", "z"],
     }
     weather_xds = weather_xds.assign_coords(coords)
 
     dims_station_time = ["station_name", "time_weather"]
+    dims_station_time_position = dims_station_time + ["cartesian_pos_label"]
     to_new_data_variables = {
-        "H20": ["H2O", dims_station_time],
+        "H2O": ["H2O", dims_station_time],
         "IONOS_ELECTRON": ["IONOS_ELECTRON", dims_station_time],
         "PRESSURE": ["PRESSURE", dims_station_time],
         "REL_HUMIDITY": ["REL_HUMIDITY", dims_station_time],
@@ -226,6 +304,15 @@ def create_weather_xds(in_file: str, ant_xds_station_name_ids: xr.DataArray):
         "WIND_DIRECTION": ["WIND_DIRECTION", dims_station_time],
         "WIND_SPEED": ["WIND_SPEED", dims_station_time],
     }
+    if has_asdm_station_position:
+        to_new_data_variables.update(
+            {
+                "NS_WX_STATION_POSITION": [
+                    "STATION_POSITION",
+                    dims_station_time_position,
+                ],
+            }
+        )
 
     to_new_coords = {
         "TIME": ["time_weather", ["time_weather"]],
@@ -234,6 +321,9 @@ def create_weather_xds(in_file: str, ant_xds_station_name_ids: xr.DataArray):
     weather_xds = convert_generic_xds_to_xradio_schema(
         generic_weather_xds, weather_xds, to_new_data_variables, to_new_coords
     )
+    weather_xds = finalize_station_position(
+        weather_xds, ant_position_with_ids, has_asdm_station_position
+    )
 
     # TODO: option to interpolate to main time
 
@@ -244,6 +334,8 @@ def create_weather_xds(in_file: str, ant_xds_station_name_ids: xr.DataArray):
     for data_var in weather_xds:
         if weather_xds.data_vars[data_var].dtype != np.float64:
             weather_xds[data_var] = weather_xds[data_var].astype(np.float64)
+    if "time_weather" in weather_xds.coords:
+        weather_xds.coords["time_weather"].attrs["type"] = "time_weather"
 
     return weather_xds
 
@@ -256,6 +348,7 @@ def correct_generic_pointing_xds(
     and tries to correct several deviations from the MSv2 specs seen in
     common test data.
     The problems fixed here include wrong dimensions:
+
     - for example transposed dimensions with respect to the MSv2 specs (output
     from CASA simulator),
     - missing/additional unexpected dimensions when some of the columns are
@@ -299,10 +392,10 @@ def correct_generic_pointing_xds(
                 and generic_pointing_xds.sizes["dir"] == 0
             ):
                 # When some direction variables are "empty" but some are populated properly
-                if "dim_2" in generic_pointing_xds[key].sizes:
+                if "dim_2" in generic_pointing_xds[data_var_name].sizes:
                     data_var_data = xr.DataArray(
-                        generic_pointing_xds[key].values,
-                        dims=generic_pointing_xds[key].dims,
+                        generic_pointing_xds[data_var_name].values,
+                        dims=generic_pointing_xds[data_var_name].dims,
                     )
                 else:
                     shape = tuple(
@@ -311,7 +404,7 @@ def correct_generic_pointing_xds(
                     ) + (2,)
                     data_var_data = xr.DataArray(
                         np.full(shape, np.nan),
-                        dims=generic_pointing_xds[key].dims,
+                        dims=generic_pointing_xds[data_var_name].dims,
                     )
                 correct_pointing_xds[data_var_name].data = data_var_data
 
@@ -363,6 +456,7 @@ def create_pointing_xds(
     generic_pointing_xds = load_generic_table(
         in_file,
         "POINTING",
+        timecols=["TIME"],
         rename_ids=subt_rename_ids["POINTING"],
         taql_where=taql_where,
     )
@@ -423,6 +517,7 @@ def prepare_generic_sys_cal_xds(generic_sys_cal_xds: xr.Dataset) -> xr.Dataset:
     sys_cal_xds dataset, as their structure differs in dimensions and order
     of dimensions.
     This function performs various prepareation steps, such as:
+
     - filter out dimensions not neeed for an individual MSv4 (SPW, FEED),
     - drop variables loaded from columns with all items set to empty array,
     - transpose the dimensions frequency,receptor
@@ -469,7 +564,7 @@ def prepare_generic_sys_cal_xds(generic_sys_cal_xds: xr.Dataset) -> xr.Dataset:
         )
     elif (
         "frequency" in generic_sys_cal_xds.sizes
-        and not "dim_3" in generic_sys_cal_xds.sizes
+        and "dim_3" not in generic_sys_cal_xds.sizes
     ):
         # because order is (...,frequency,receptor), when frequency is missing
         # receptor can get wrongly labeled as frequency
@@ -496,6 +591,7 @@ def prepare_generic_sys_cal_xds(generic_sys_cal_xds: xr.Dataset) -> xr.Dataset:
 
 def create_system_calibration_xds(
     in_file: str,
+    spectral_window_id: int,
     main_xds_frequency: xr.DataArray,
     ant_xds: xr.DataArray,
     sys_cal_interp_time: Union[xr.DataArray, None] = None,
@@ -507,9 +603,11 @@ def create_system_calibration_xds(
     ----------
     in_file: str
         Input MS name.
+    spectral_window_id: int
+        SPW ID from MSv2
     main_xds_frequency: xr.DataArray
         frequency array of the main xds (MSv4), containing among other things
-        spectral_window_id and measures metadata
+        the measures metadata
     ant_xds : xr.Dataset
         The antenna_xds that has information such as names, stations, etc., for coordinates
     sys_cal_interp_time: Union[xr.DataArray, None] = None,
@@ -521,11 +619,11 @@ def create_system_calibration_xds(
         System calibration Xarray Dataset.
     """
 
-    spectral_window_id = main_xds_frequency.attrs["spectral_window_id"]
     try:
         generic_sys_cal_xds = load_generic_table(
             in_file,
             "SYSCAL",
+            timecols=["TIME"],
             rename_ids=subt_rename_ids["SYSCAL"],
             taql_where=(
                 f" where (SPECTRAL_WINDOW_ID = {spectral_window_id})"
@@ -586,11 +684,9 @@ def create_system_calibration_xds(
             "frequency_system_cal": generic_sys_cal_xds.coords["frequency"].data
         }
         sys_cal_xds = sys_cal_xds.assign_coords(frequency_coord)
-        frequency_measure = {
-            "type": main_xds_frequency.attrs["type"],
-            "units": main_xds_frequency.attrs["units"],
-            "observer": main_xds_frequency.attrs["observer"],
-        }
+        frequency_measure = make_spectral_coord_measure_attrs(
+            main_xds_frequency.attrs["units"], main_xds_frequency.attrs["observer"]
+        )
         sys_cal_xds.coords["frequency_system_cal"].attrs.update(frequency_measure)
 
     sys_cal_xds = rename_and_interpolate_to_time(
@@ -603,3 +699,117 @@ def create_system_calibration_xds(
             sys_cal_xds[data_var] = sys_cal_xds[data_var].astype(np.float64)
 
     return sys_cal_xds
+
+
+def create_phased_array_xds(
+    in_file: str,
+    antenna_names: list[str],
+    receptor_label: list[str],
+    polarization_type: ArrayLike,
+) -> Optional[xr.Dataset]:
+    """
+    Create an Xarray Dataset containing phased array information.
+
+    Parameters
+    ----------
+    in_file : str
+        Path to the input MSv2.
+    antenna_names: DataArray or Sequence[str]
+        Content of the antenna_name coordinate of the antenna_xds.
+    receptor_label: DataArray or Sequence[str]
+        Content of the receptor_label coordinate of the antenna_xds. Used to
+        label the corresponding axis of ELEMENT_FLAG.
+    polarization_type: DataArray or ArrayLike
+        Contents of the polarization_type coordinate of the antenna_xds.
+        Array-like of shape (num_antennas, 2) containing the polarization
+        hands for each antenna.
+
+    Returns
+    ----------
+        xr.Dataset or None: If the input MS contains a PHASED_ARRAY table,
+           returns the Xarray Dataset containing the phased array information.
+           Otherwise, return None.
+    """
+
+    def extract_data(dataarray_or_sequence):
+        if hasattr(dataarray_or_sequence, "data"):
+            return dataarray_or_sequence.data.tolist()
+        return dataarray_or_sequence
+
+    antenna_names = extract_data(antenna_names)
+    receptor_label = extract_data(receptor_label)
+    polarization_type = extract_data(polarization_type)
+
+    # NOTE: We cannot use the dimension renaming option of `load_generic_table`
+    # here, because it leads to a dimension name collision. This is caused by
+    # the presence of two dimensions of size 3 in multiple arrays.
+    # Instead, we do the renaming manually below.
+    try:
+        raw_xds = load_generic_table(
+            in_file,
+            "PHASED_ARRAY",
+            # Some MSes carry COORDINATE_SYSTEM as a copy of COORDINATE_AXES
+            # due to a past ambiguity on the PHASED_ARRAY schema
+            ignore=["COORDINATE_SYSTEM", "ANTENNA_ID"],
+        )
+    except ValueError:
+        return None
+
+    # Defend against empty PHASED_ARRAY table.
+    # The test MS "AA2-Mid-sim_00000.ms" has that problem.
+    required_keys = {"COORDINATE_AXES", "ELEMENT_OFFSET", "ELEMENT_FLAG"}
+    if not all(k in raw_xds for k in required_keys):
+        return None
+
+    def msv4_measure(raw_name: str) -> dict:
+        coldesc = raw_xds.attrs["other"]["msv2"]["ctds_attrs"]["column_descriptions"]
+        return column_description_casacore_to_msv4_measure(coldesc[raw_name])
+
+    def make_data_variable(raw_name: str, dim_names: list[str]) -> xr.DataArray:
+        da = raw_xds[raw_name]
+        da = xr.DataArray(da.data, dims=tuple(dim_names))
+        return da.assign_attrs(msv4_measure(raw_name))
+
+    raw_datavar_names_and_dims = [
+        (
+            "COORDINATE_AXES",
+            ("antenna_name", "cartesian_pos_label_local", "cartesian_pos_label"),
+        ),
+        ("ELEMENT_OFFSET", ("antenna_name", "cartesian_pos_label_local", "element_id")),
+        ("ELEMENT_FLAG", ("antenna_name", "receptor_label", "element_id")),
+    ]
+
+    data_vars = {
+        name: make_data_variable(name, dims)
+        for name, dims in raw_datavar_names_and_dims
+    }
+    data_vars["COORDINATE_AXES"].attrs = {
+        "type": "rotation_matrix",
+        "units": "dimensionless",
+    }
+    # Remove the "frame" attribute if it exists, because ELEMENT_OFFSET is
+    # defined in a station-local frame for which no standard name exists
+    data_vars["ELEMENT_OFFSET"].attrs.pop("frame", None)
+    data_vars["ELEMENT_OFFSET"].attrs.update(
+        {
+            "coordinate_system": "topocentric",
+            "origin": "ANTENNA_POSITION",
+        }
+    )
+
+    num_elements = data_vars["ELEMENT_OFFSET"].sizes["element_id"]
+
+    data_vars = {"PHASED_ARRAY_" + key: val for key, val in data_vars.items()}
+    coords = {
+        "antenna_name": antenna_names,
+        "element_id": np.arange(num_elements),
+        "receptor_label": receptor_label,
+        "polarization_type": (
+            ("antenna_name", "receptor_label"),
+            polarization_type,
+        ),
+        "cartesian_pos_label": ["x", "y", "z"],
+        "cartesian_pos_label_local": ["p", "q", "r"],
+    }
+    attrs = {"type": "phased_array"}
+    return xr.Dataset(data_vars, coords, attrs)
