@@ -1,7 +1,9 @@
+import builtins
 import dataclasses
 import typing
 import inspect
 import functools
+import warnings
 
 import xarray
 import numpy
@@ -14,6 +16,8 @@ from xradio.schema import (
     xarray_dataclass_to_dataset_schema,
     xarray_dataclass_to_dict_schema,
 )
+from xradio.schema.dataclass import value_schema
+from xradio.schema.metamodel import AttrSchemaRef, ValueSchema
 
 
 @dataclasses.dataclass
@@ -73,7 +77,7 @@ class SchemaIssues(Exception):
     in one go.
     """
 
-    issues: [SchemaIssue]
+    issues: list[SchemaIssue]
     """List of issues found"""
 
     def __init__(self, issues=None):
@@ -143,8 +147,8 @@ def check_array(
     Check whether an xarray DataArray conforms to a schema
 
     :param array: DataArray to check
-    :param schema: Schema to check against (possibly as :py:class:`AsDataset`)
-    :returns: List of :py:class:`SchemaIssue`s found
+    :param schema: Schema to check against
+    :returns: :py:class:`SchemaIssues` found
     """
 
     # Check that this is actually a DataArray
@@ -173,14 +177,16 @@ def check_array(
 
 
 def check_dataset(
-    dataset: xarray.Dataset, schema: typing.Union[type, metamodel.DatasetSchema]
+    dataset: xarray.Dataset,
+    schema: typing.Union[type, metamodel.DatasetSchema],
+    allow_superflous_dims: typing.Set[str] = frozenset(),
 ) -> SchemaIssues:
     """
     Check whether an xarray DataArray conforms to a schema
 
     :param array: DataArray to check
-    :param schema: Schema to check against (possibly as :py:class:`AsDataArray` dataclass)
-    :returns: List of :py:class:`SchemaIssue`s found
+    :param schema: Schema to check against
+    :returns: :py:class:`SchemaIssues` found
     """
 
     # Check that this is actually a Dataset
@@ -196,7 +202,12 @@ def check_dataset(
         )
 
     # Check dimensions. Order does not matter on datasets
-    issues = check_dimensions(dataset.dims, schema.dimensions, check_order=False)
+    issues = check_dimensions(
+        dataset.dims,
+        schema.dimensions,
+        check_order=False,
+        allow_superflous=allow_superflous_dims,
+    )
 
     # Check attributes
     issues += check_attributes(dataset.attrs, schema.attributes)
@@ -211,7 +222,10 @@ def check_dataset(
 
 
 def check_dimensions(
-    dims: [str], expected: [[str]], check_order: bool = True
+    dims: [str],
+    expected: [[str]],
+    check_order: bool = True,
+    allow_superflous: typing.Set[str] = frozenset(),
 ) -> SchemaIssues:
     """
     Check whether a dimension list conforms to a schema
@@ -219,7 +233,7 @@ def check_dimensions(
     :param array: Dimension list to check
     :param schema: Expected possibilities for dimension list
     :param check_order: Whether to check order of dimensions
-    :returns: List of :py:class:`SchemaIssue`s found
+    :returns: :py:class:`SchemaIssues` found
     """
 
     # Find a dimension list that matches
@@ -230,7 +244,7 @@ def check_dimensions(
         exp_dims_set = set(exp_dims)
 
         # No match? Continue, but take note of best match
-        if exp_dims_set != dims_set:
+        if exp_dims_set - allow_superflous != dims_set - allow_superflous:
             diff = len(dims_set.symmetric_difference(exp_dims_set))
             if best is None or diff < best_diff:
                 best = exp_dims_set
@@ -253,7 +267,7 @@ def check_dimensions(
         )
 
     # Dimensionality not supported - try to give a helpful suggestion
-    hint_remove = [f"'{hint}'" for hint in dims_set - best]
+    hint_remove = [f"'{hint}'" for hint in (dims_set - best) - allow_superflous]
     hint_add = [f"'{hint}'" for hint in best - dims_set]
     if hint_remove and hint_add:
         message = f"Unexpected coordinates, replace {','.join(hint_remove)} by {','.join(hint_add)}?"
@@ -281,10 +295,11 @@ def check_dtype(dtype: numpy.dtype, expected: [numpy.dtype]) -> SchemaIssues:
 
     :param dtype: Numeric type to check
     :param schema: Expected possibilities for dtype
-    :returns: List of :py:class:`SchemaIssue`s found
+    :returns: :py:class:`SchemaIssues` found
     """
 
-    for exp_dtype in expected:
+    for exp_dtype_str in expected:
+        exp_dtype = numpy.dtype(exp_dtype_str)
         # If the expected dtype has no size (e.g. "U", a.k.a. a string of
         # arbitrary length), we don't check itemsize, only kind.
         if (
@@ -301,7 +316,7 @@ def check_dtype(dtype: numpy.dtype, expected: [numpy.dtype]) -> SchemaIssues:
             SchemaIssue(
                 path=[("dtype", None)],
                 message="Wrong numpy dtype",
-                found=dtype,
+                found=dtype.str,
                 expected=list(expected),
             )
         ]
@@ -318,35 +333,28 @@ def check_attributes(
 
     :param attrs: Dictionary of attributes
     :param attrs_schema: Expected schemas
-    :returns: List of :py:class:`SchemaIssue`s found
+    :returns: :py:class:`SchemaIssues` found
     """
 
     issues = SchemaIssues()
     for attr_schema in attrs_schema:
-        # Attribute missing? Note that a value of "None" is equivalent for the
-        # purpose of the check
+        # Attribute missing is equivalent to a value of "None" is
+        # equivalent for the purpose of the check
         val = attrs.get(attr_schema.name)
         if val is None:
             if not attr_schema.optional:
-                # Get options
-                if typing.get_origin(attr_schema.typ) is typing.Union:
-                    options = typing.get_args(attr_schema.typ)
-                else:
-                    options = [attr_schema.typ]
-
                 issues.add(
                     SchemaIssue(
                         path=[(attr_kind, attr_schema.name)],
-                        message=f"Required attribute {attr_schema.name} is missing!",
-                        expected=options,
+                        message="Non-optional attribute is missing!",
+                        found=None,
+                        expected=[attr_schema.type],
                     )
                 )
             continue
 
-        # Check attribute value
-        issues += _check_value_union(val, attr_schema.typ).at_path(
-            attr_kind, attr_schema.name
-        )
+        # Check actual value
+        issues += _check_value(val, attr_schema).at_path(attr_kind, attr_schema.name)
 
     # Extra attributes are always okay
 
@@ -359,7 +367,7 @@ def check_data_vars(
     data_var_kind: str,
 ) -> SchemaIssues:
     """
-    Check whether an data variable set conforms to a schema
+    Check whether a data variable set conforms to a schema
 
     As data variables are data arrays, this will recurse into checking the
     array schemas
@@ -367,22 +375,21 @@ def check_data_vars(
     :param data_vars: Dictionary(-like) of data_varinates
     :param data_vars_schema: Expected schemas
     :param datavar_kind: Either 'coords' or 'data_vars'
-    :returns: List of :py:class:`SchemaIssue`s found
+    :returns: :py:class:`SchemaIssues` found
     """
 
     assert data_var_kind in ["coords", "data_vars"]
 
     issues = SchemaIssues()
     for data_var_schema in data_vars_schema:
-
-        allow_mutiple_versions = False
+        allow_multiple_versions = False
         for attr in data_var_schema.attributes:
             if hasattr(attr, "name"):
-                if attr.name == "allow_mutiple_versions":
-                    allow_mutiple_versions = attr.default
+                if attr.name == "allow_multiple_versions":
+                    allow_multiple_versions = attr.default
 
         data_vars_names = []
-        if allow_mutiple_versions:
+        if allow_multiple_versions:
             for data_var_name in data_vars:
                 if data_var_schema.name in data_var_name:
                     data_vars_names.append(data_var_name)
@@ -427,6 +434,14 @@ def check_data_vars(
 def check_dict(
     dct: dict, schema: typing.Union[type, metamodel.DictSchema]
 ) -> SchemaIssues:
+    """
+    Check whether a dictionary conforms to a schema
+
+    :param dct: Dictionary to check
+    :param schema: Dictionary schema to check against
+    :returns: :py:class:`SchemaIssues` found
+    """
+
     # Check that this is actually a dictionary
     if not isinstance(dct, dict):
         raise TypeError(f"check_dict: Expected dictionary, but got {type(dct)}!")
@@ -439,125 +454,189 @@ def check_dict(
     return check_attributes(dct, schema.attributes, attr_kind="")
 
 
-def _check_value(val, ann):
+def _check_value(val: typing.Any, schema: metamodel.ValueSchema):
     """
     Check whether value satisfies annotation
 
     If the annotation is a data array or dataset schema, it will be checked.
 
     :param val: Value to check
-    :param ann: Type annotation of value
+    :param schema: Schema of value
     :returns: Schema issues
     """
 
+    # Unspecified?
+    if schema.type is None:
+        return SchemaIssues()
+
+    # Optional?
+    if schema.optional and val is None:
+        return SchemaIssues()
+
     # Is supposed to be a data array?
-    if bases.is_dataarray_schema(ann):
+    if schema.type == "dataarray":
         # Attempt to convert dictionaries automatically
         if isinstance(val, dict):
             try:
                 val = xarray.DataArray.from_dict(val)
             except ValueError as e:
+                expected = [DataArray]
+                if schema.optional:
+                    expected.append(type(None))
                 return SchemaIssues(
                     [
                         SchemaIssue(
-                            path=[], message=str(e), expected=[ann], found=type(val)
+                            path=[], message=str(e), expected=expected, found=type(val)
                         )
                     ]
                 )
             except TypeError as e:
+                expected = [DataArray]
+                if schema.optional:
+                    expected.append(type(None))
                 return SchemaIssues(
                     [
                         SchemaIssue(
-                            path=[], message=str(e), expected=[ann], found=type(val)
+                            path=[], message=str(e), expected=expected, found=type(val)
                         )
                     ]
                 )
 
         if not isinstance(val, xarray.DataArray):
             # Fall through to plain type check
-            ann = xarray.DataArray
+            type_to_check = xarray.DataArray
         else:
-            return check_array(val, ann)
-
-    # Is supposed to be a dataset?
-    if bases.is_dataset_schema(ann):
-        # Attempt to convert dictionaries automatically
-        if isinstance(val, dict):
-            try:
-                val = xarray.Dataset.from_dict(val)
-            except ValueError as e:
-                return SchemaIssues(
-                    [
-                        SchemaIssue(
-                            path=[], message=str(t), expected=[ann], found=type(val)
-                        )
-                    ]
-                )
-        if not isinstance(val, xarray.Dataset):
-            # Fall through to plain type check
-            ann = xarray.Dataset
-        else:
-            return check_dataset(val, ann)
+            return check_array(val, schema.array_schema)
 
     # Is supposed to be a dictionary?
-    if bases.is_dict_schema(ann):
+    elif schema.type == "dict":
         if not isinstance(val, dict):
             # Fall through to plain type check
-            ann = dict
+            type_to_check = dict
         else:
-            return check_dict(val, ann)
+            return check_dict(val, schema.dict_schema)
+
+    elif schema.type == "list[str]":
+        type_to_check = typing.List[str]
+    elif schema.type in ["bool", "str", "int", "float"]:
+        type_to_check = getattr(builtins, schema.type)
+    else:
+        raise ValueError(f"Invalid typ_name in schema: {schema.type}")
 
     # Otherwise straight type check using typeguard
     try:
-        check_type(val, ann)
+        check_type(val, type_to_check)
     except TypeCheckError as t:
+        expected = [type_to_check]
+        if schema.optional:
+            expected.append(type(None))
         return SchemaIssues(
-            [SchemaIssue(path=[], message=str(t), expected=[ann], found=type(val))]
+            [SchemaIssue(path=[], message=str(t), expected=expected, found=type(val))]
+        )
+
+    # List of literals given?
+    if schema.literal is not None:
+        for lit in schema.literal:
+            if val == lit:
+                return SchemaIssues()
+        return SchemaIssues(
+            [
+                SchemaIssue(
+                    path=[],
+                    message=f"Disallowed literal value!",
+                    expected=schema.literal,
+                    found=val,
+                )
+            ]
         )
 
     return SchemaIssues()
 
 
-def _check_value_union(val, ann):
+_DATASET_TYPES = {}
+
+
+def register_dataset_type(schema: metamodel.DatasetSchema):
     """
-    Check whether value satisfies annotations, including union types
+    Registers the given schema for usage with :py:meth:`check_datatree`
 
-    If the annotation is a data array or dataset schema, it will be checked.
+    This looks for a ``type`` attribute in the dataset schema, which
+    must have a :py:class:`typing.Literal` type annotation specifying
+    the type name of the dataset
 
-    :param val: Value to check
-    :param ann: Type annotation of value
-    :returns: Schema issues
+    :param schema: Schema to register
     """
 
-    if ann is None or ann is inspect.Signature.empty:
-        return SchemaIssues()
+    # Find type attribute
+    for attr in schema.attributes:
+        if attr.name != "type":
+            continue
 
-    # Account for union types (this especially catches "Optional")
-    if typing.get_origin(ann) is typing.Union:
-        options = typing.get_args(ann)
-    else:
-        options = [ann]
+        # Type should be a kind of literal
+        if attr.literal is None:
+            warnings.warn(
+                f"In dataset schema {schema.schema_name}:"
+                'Attribute "type" should be a literal!'
+            )
+            continue
 
-    # Go through options, try to find one without issues
-    args_issues = None
-    okay = False
-    for option in options:
-        arg_issues = _check_value(val, option)
-        # We can immediately return if we find no issues with
-        # some schema check
-        if not arg_issues:
-            return SchemaIssues()
-        if args_issues is None:
-            args_issues = arg_issues
+        # Register type names
+        for typ in attr.literal:
+            assert isinstance(typ, str), (
+                f"In dataset schema {schema.schema_name}:"
+                'Attribute "type" should be a literal giving '
+                "names of schema!"
+            )
+            _DATASET_TYPES[typ] = schema
 
-        # Crude merging of expected options (for "unexpected type")
-        elif len(args_issues) == 1 and len(arg_issues) == 1:
-            args_issues[0].expected += arg_issues[0].expected
 
-    # Return representative issues list
-    if not args_issues:
-        raise ValueError("Empty union set?")
-    return args_issues
+def check_datatree(
+    datatree: xarray.DataTree,
+):
+    """
+    Check datatree for schema conformance
+
+    This is the case if each contained :py:class:`~xarray.Dataset` conforms to a schema
+    registed with Xradio.
+    This works by looking for a ``type`` attribute in the :py:class:`~xarray.Dataset`, which
+    must have a :py:class:`typing.Literal` type annotation specifying
+    the name of the dataset schema.
+
+    :param datatree: Data to check for schema conformance
+    """
+
+    # Loop through all groups in datatree
+    issues = SchemaIssues()
+    for xds_name in datatree.groups:
+        # Ignore any leaf without data
+        node = datatree[xds_name]
+        if not node.has_data:
+            continue
+
+        # Look up schema
+        typ = node.attrs.get("type")
+        schema = _DATASET_TYPES.get(typ)
+        if schema is None:
+            issues.add(
+                SchemaIssue(
+                    [("", xds_name)],
+                    message="Unknown dataset type!",
+                    found=typ,
+                    expected=None,
+                )
+            )
+            continue
+
+        # Determine dimensions inherited from parent
+        # (they might show up as "superflous" for the child schema)
+        parent_dims = frozenset()
+        if node.parent is not None:
+            parent_dims = set(node.parent.dims)
+
+        # Check schema
+        issues += check_dataset(node.dataset, schema, parent_dims).at_path("", xds_name)
+
+    return issues
 
 
 def schema_checked(fn, check_parameters: bool = True, check_return: bool = True):
@@ -586,7 +665,7 @@ def schema_checked(fn, check_parameters: bool = True, check_return: bool = True)
     @functools.wraps(fn)
     def _check_fn(*args, **kwargs):
         # Hide this function in pytest tracebacks
-        __tracebackhide__ = True
+        # __tracebackhide__ = True
 
         # Bind parameters, collect (potential) issues
         bound = signature.bind(*args, **kwargs)
@@ -596,7 +675,15 @@ def schema_checked(fn, check_parameters: bool = True, check_return: bool = True)
                 continue
 
             # Get annotation
-            issues += _check_value_union(val, anns.get(arg)).at_path(arg)
+            vschema = value_schema(anns.get(arg), "function", arg)
+            pseudo_attr_schema = AttrSchemaRef(
+                name=arg,
+                **{
+                    fld.name: getattr(vschema, fld.name)
+                    for fld in dataclasses.fields(ValueSchema)
+                },
+            )
+            issues += _check_value(val, pseudo_attr_schema).at_path(arg)
 
         # Any issues found? raise
         issues.expect()
@@ -606,7 +693,15 @@ def schema_checked(fn, check_parameters: bool = True, check_return: bool = True)
 
         # Check return
         if check_return:
-            issues = _check_value_union(val, signature.return_annotation)
+            vschema = value_schema(anns.get(arg), "function", "return")
+            pseudo_attr_schema = AttrSchemaRef(
+                name="return",
+                **{
+                    fld.name: getattr(vschema, fld.name)
+                    for fld in dataclasses.fields(ValueSchema)
+                },
+            )
+            issues = _check_value(val, pseudo_attr_schema)
             issues.at_path("return").expect()
 
         # Check return value
